var Csv, Excel, Promise, Reader, _, debugLog, fs, iconv, path,
  bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

_ = require('underscore');

Csv = require('csv');

path = require('path');

Promise = require('bluebird');

iconv = require('iconv-lite');

fs = Promise.promisifyAll(require('fs'));

Excel = require('exceljs');

debugLog = console.log;

debugLog = _.noop;

Reader = (function() {
  function Reader(options1) {
    this.options = options1 != null ? options1 : {};
    this._readXlsx = bind(this._readXlsx, this);
    this._readCsv = bind(this._readCsv, this);
    this._parseCsv = bind(this._parseCsv, this);
    this.read = bind(this.read, this);
    debugLog("READER::options:", JSON.stringify(this.options));
    this.options.encoding = this.options.encoding || 'utf-8';
    this.options.csvDelimiter = this.options.csvDelimiter || ',';
    this.header = null;
    this.rows = [];
    if (!iconv.encodingExists(this.options.encoding)) {
      throw new Error('Encoding does not exist: ' + this.options.encoding);
    }
  }

  Reader.prototype.read = function(file) {
    debugLog("READER::stream file %s", file);
    this.inputStream = fs.createReadStream(file);
    if (this.options.importFormat === 'xlsx') {
      return this._readXlsx(this.inputStream);
    } else {
      return this._readCsv(this.inputStream);
    }
  };

  Reader.prototype._mapRow = function(header, row) {
    var res;
    res = {};
    header.forEach(function(item, index) {
      return res[item] = row[index];
    });
    return res;
  };

  Reader.prototype._parseCsv = function(csv, delimiter, encoding) {
    var options;
    options = {
      columns: true,
      delimiter: delimiter,
      skip_empty_lines: true
    };
    if (csv instanceof Buffer) {
      csv = this._decode(csv, encoding);
    }
    return new Promise(function(resolve, reject) {
      return Csv.parse(csv, options, function(err, output) {
        if (err) {
          return reject(err);
        } else {
          return resolve(output);
        }
      });
    });
  };

  Reader.prototype._readCsv = function(stream) {
    return new Promise((function(_this) {
      return function(resolve, reject) {
        var buffers;
        buffers = [];
        stream.on('data', function(buffer) {
          return buffers.push(buffer);
        });
        stream.on('error', function(err) {
          return reject(err);
        });
        return stream.on('end', function() {
          var buffer;
          buffer = Buffer.concat(buffers);
          return _this._parseCsv(buffer, _this.options.csvDelimiter, _this.options.encoding).then(function(parsed) {
            return resolve(parsed);
          })["catch"](function(err) {
            return reject(err);
          });
        });
      };
    })(this));
  };

  Reader.prototype._readXlsx = function(stream) {
    var workbook;
    workbook = new Excel.Workbook();
    return workbook.xlsx.read(stream).then((function(_this) {
      return function(workbook) {
        var header, rows, worksheet;
        header = null;
        rows = [];
        worksheet = workbook.getWorksheet(1);
        worksheet.eachRow(function(row) {
          var rowValues, rowVaues;
          rowValues = row.values;
          rowValues.shift();
          rowVaues = _.map(rowValues, function(item) {
            if ((item == null) || _.isObject(item)) {
              item = "";
            }
            return String(item);
          });
          if (!header) {
            return header = rowVaues;
          } else {
            return rows.push(_this._mapRow(header, rowVaues));
          }
        });
        return rows;
      };
    })(this));
  };

  Reader.prototype._decode = function(buffer, encoding) {
    debugLog("READER:decode from %s", encoding);
    if (encoding === 'utf-8') {
      return buffer.toString();
    }
    return iconv.decode(buffer, encoding);
  };

  return Reader;

})();

module.exports = Reader;
