/* ===========================================================
# sphere-product-type-json-generator - v3.0.0
# ==============================================================
# Copyright (c) 2014 Sven Müller
# Licensed under the MIT license.
*/
var ExtendedLogger, ProductTypeImport, ProductTypeImporter, ProjectCredentialsConfig, Promise, _, package_json, path, ref;

_ = require('underscore');

path = require('path');

Promise = require('bluebird');

ProductTypeImport = require('sphere-product-type-import');

ref = require('sphere-node-utils'), ExtendedLogger = ref.ExtendedLogger, ProjectCredentialsConfig = ref.ProjectCredentialsConfig;

package_json = require('../package.json');

ProductTypeImporter = (function() {
  function ProductTypeImporter() {}


  /*
  Prepare for importing, initialize used modules
   */

  ProductTypeImporter.prototype.init = function(argv) {
    var logPath;
    logPath = path.join(argv.logDir || '.', package_json.name + ".log");
    this.logOptions = {
      name: package_json.name + "-" + package_json.version,
      silent: !!argv.logSilent,
      streams: [
        {
          level: 'error',
          stream: process.stderr
        }, {
          level: argv.logLevel || 'info',
          path: logPath
        }
      ]
    };
    this.logger = new ExtendedLogger({
      additionalFields: {
        'project_key': argv.projectKey
      },
      logConfig: this.logOptions
    });
    if (argv.logSilent) {
      this.logger.bunyanLogger.trace = function() {};
      this.logger.bunyanLogger.debug = function() {};
    }
    return this._ensureCredentials(argv).then((function(_this) {
      return function(credentials) {
        _this.sphereImporter = new ProductTypeImport["default"](_this.logger, credentials);
        return Promise.resolve(_this);
      };
    })(this));
  };


  /*
  Create sphere credentials config from command line arguments
   */

  ProductTypeImporter.prototype._ensureCredentials = function(argv) {
    var credentialsPromise;
    credentialsPromise = null;
    if (argv.accessToken) {
      credentialsPromise = Promise.resolve({
        config: {
          project_key: argv.projectKey
        },
        access_token: argv.accessToken
      });
    } else {
      credentialsPromise = ProjectCredentialsConfig.create().then(function(credentials) {
        var config;
        config = credentials.enrichCredentials({
          project_key: argv.projectKey
        });
        if (argv.clientId) {
          config.client_id = argv.clientId;
        }
        if (argv.clientSecret) {
          config.client_secret = argv.clientSecret;
        }
        return {
          config: config
        };
      });
    }
    return credentialsPromise.then(function(credentials) {
      var options;
      options = _.extend(credentials, {
        timeout: argv.timeout,
        user_agent: package_json.name + " - " + package_json.version
      });
      if (argv.sphereHost) {
        options.host = argv.sphereHost;
      }
      if (argv.sphereProtocol) {
        options.protocol = argv.sphereProtocol;
      }
      if (argv.sphereAuthHost) {
        options.oauth_host = argv.sphereAuthHost;
        options.rejectUnauthorized = false;
      }
      if (argv.sphereAuthProtocol) {
        options.oauth_protocol = argv.sphereAuthProtocol;
      }
      return {
        sphereClientConfig: options
      };
    });
  };


  /*
  Import product types using sphere product import tool
   */

  ProductTypeImporter.prototype["import"] = function(data) {
    console.log("Importing product types to sphere");
    return Promise.map(data.productTypes, (function(_this) {
      return function(productType) {
        console.log("Importing product type with name:", productType.name);
        return _this.sphereImporter.importProductType(productType).then(function(res) {
          console.log("Imported product type", res.name);
          return Promise.resolve(res);
        })["catch"](function(e) {
          console.error("Oops, something went wrong when importing product type " + productType.name + " to sphere");
          return Promise.reject(e);
        });
      };
    })(this), {
      concurrency: 5
    });
  };

  return ProductTypeImporter;

})();

module.exports = ProductTypeImporter;
