/* ===========================================================
# sphere-product-type-json-generator - v3.0.0
# ==============================================================
# Copyright (c) 2014 Sven Müller
# Licensed under the MIT license.
*/
var ProjectCredentialsConfig, Promise, SphereClient, _, argv, fs, package_json;

_ = require('underscore');

Promise = require('bluebird');

fs = Promise.promisifyAll(require('fs'));

SphereClient = require('sphere-node-sdk').SphereClient;

ProjectCredentialsConfig = require('sphere-node-utils').ProjectCredentialsConfig;

package_json = require('../package.json');

argv = require('optimist').usage('Usage: $0 --projectKey [key] --clientId [id] --clientSecret [secret] --source [path]').describe('projectKey', 'your SPHERE.IO project-key').describe('clientId', 'your OAuth client id for the SPHERE.IO API').describe('clientSecret', 'your OAuth client secret for the SPHERE.IO API').describe('source', 'path to JSON file or folder (in case of folder, all JSON files in that folder will be used)').demand(['projectKey', 'source']).argv;

ProjectCredentialsConfig.create().then(function(config) {
  var client, credentials, processFile;
  credentials = config.enrichCredentials({
    project_key: argv.projectKey,
    client_id: argv.clientId,
    client_secret: argv.clientSecret
  });
  client = new SphereClient({
    config: credentials,
    user_agent: package_json.name + " - " + package_json.version
  });
  processFile = function(path) {
    console.log("Processing file " + path);
    return fs.readFileAsync(path, {
      encoding: 'utf-8'
    }).then(function(content) {
      var payload;
      payload = JSON.parse(content);
      return client.productTypes.create(payload);
    });
  };
  return fs.exists(argv.source, function(exists) {
    if (exists) {
      return fs.statAsync(argv.source).then(function(stats) {
        if (stats.isFile()) {
          return processFile(argv.source);
        } else if (stats.isDirectory()) {
          console.log("About to read all files in directory " + argv.source);
          return fs.readdirAsync(argv.source).then(function(files) {
            var jsonFiles;
            jsonFiles = _.filter(files, function(file) {
              return file.match(/\.json/);
            });
            console.log("Processing " + (_.size(jsonFiles)) + " files in directory " + argv.source);
            return Promise.map(jsonFiles, function(file) {
              return processFile(argv.source + "/" + file);
            }, {
              concurrency: 1
            });
          });
        } else {
          return Promise.reject("Given path is not a file nor a directory " + argv.source);
        }
      }).then(function() {
        console.log('Product Types successfully posted to SPHERE.IO');
        return process.exit(0);
      })["catch"](function(e) {
        console.error("Oops, something went wrong: " + e);
        if (e.body) {
          console.error("Body: %j", e.body);
        }
        return process.exit(1);
      });
    } else {
      console.error("Could not find " + argv.source + " path");
      return process.exit(1);
    }
  });
});
